# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2013 Mariano Ruiz <mrsarm@gmail.com>
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
import decimal_precision as dp

class account_invoice(osv.osv):
    _inherit = "account.invoice"

    def _amount_all_co_currency(self, cr, uid, ids, field_name, arg, context={}):
        res = {}
        for invoice in self.browse(cr, uid, ids, context=context):
            if invoice.company_id.currency_id == invoice.currency_id:
                res[invoice.id] = {
                    'amount_co_currency_untaxed': invoice.amount_untaxed,
                    'amount_co_currency_tax': invoice.amount_tax,
                    'amount_co_currency_total': invoice.amount_total,
                }
            else:
                move_obj = self.pool.get("account.move")
                res[invoice.id] = {
                    'amount_co_currency_untaxed': 0.0,
                    'amount_co_currency_tax': 0.0,
                    'amount_co_currency_total': 0.0,
                }
                for invoice_line in invoice.invoice_line:
                    res[invoice.id]['amount_co_currency_untaxed'] += move_obj._get_amount_co_currency(
                            cr, uid, invoice.move_id, invoice_line.price_subtotal, context=context)
                for tax_line in invoice.tax_line:
                    res[invoice.id]['amount_co_currency_tax'] += move_obj._get_amount_co_currency(
                            cr, uid, invoice.move_id, tax_line.amount, context=context)
                res[invoice.id]['amount_co_currency_total'] = \
                        res[invoice.id]['amount_co_currency_tax'] + res[invoice.id]['amount_co_currency_untaxed']
        return res

    _columns = {
        # Creation Date field was added to make it sortable        
        'create_date': fields.datetime('Date Created', readonly=True),

        'amount_co_currency_untaxed': fields.function(_amount_all_co_currency,
                digits_compute=dp.get_precision('Account'), string='Untaxed in Company Currency', method=True, multi=True),
        'amount_co_currency_tax': fields.function(_amount_all_co_currency,
                digits_compute=dp.get_precision('Account'), string='Tax in Company Currency', method=True, multi=True),
        'amount_co_currency_total': fields.function(_amount_all_co_currency,
                digits_compute=dp.get_precision('Account'), string='Total in Company Currency', method=True, multi=True),
    }

account_invoice()

class account_invoice_tax(osv.osv):
    _inherit = "account.invoice.tax"

    def _amount_all_co_currency(self, cr, uid, ids, field_name, arg, context={}):
        res = {}
        for tax in self.browse(cr, uid, ids, context=context):
            if tax.company_id.currency_id == tax.invoice_id.currency_id:
                res[tax.id] = {
                    'base_co_currency': tax.base,
                    'amount_co_currency': tax.amount,
                }
            else:
                move_obj = self.pool.get("account.move")
                res[tax.id] = {
                    'base_co_currency': move_obj._get_amount_co_currency(
                            cr, uid, tax.invoice_id.move_id, tax.base, context=context),
                    'amount_co_currency': move_obj._get_amount_co_currency(
                            cr, uid, tax.invoice_id.move_id, tax.amount, context=context),
                }
        return res

    _columns = {
        'base_co_currency': fields.function(_amount_all_co_currency,
                digits_compute=dp.get_precision('Account'), string='Base in Company Currency', method=True, multi=True),
        'amount_co_currency': fields.function(_amount_all_co_currency,
                digits_compute=dp.get_precision('Account'), string='Amount in Company Currency', method=True, multi=True),
    }

account_invoice_tax()
