# -*- coding: utf-8 -*-
##############################################################################
#    
#    Copyright (C) 2011 Agile Business Group sagl (<http://www.agilebg.com>)
#    Copyright (C) 2011 Domsense srl (<http://www.domsense.com>)
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
from openerp.tools.translate import _

class account_invoice(osv.osv):
    _inherit = "account.invoice"

    _columns = {
            'internal_number': fields.char('Invoice Number', size=32,
                    readonly=True, states={'draft':[('readonly',False)]}),
        }

    def _must_validate_internal_number(self, cr, uid, journal_id=None, context={}):
        """
        Return a Stock Location from ir.property values, or None.
        """
        property_obj = self.pool.get('ir.property')
        prop_ids = property_obj.search(cr, uid, [('name','=','account_invoice_validate_internal_number')],context=context)
        if len(prop_ids)>0:
            return property_obj.read(cr,uid,prop_ids[0],['value_text'],context=context)['value_text'].lower() == "true"
        return False

    def _validate_internal_number(self, cr, uid, internal_number, journal_id, context={}):
        errors = []
        sequence = self.pool.get("account.journal").browse(cr, uid, journal_id, context=context).sequence_id
        num_chars = len(sequence.prefix or "") + len(sequence.suffix or "") + sequence.padding
        if len(internal_number)!=num_chars:
            errors.append(_('Force Number field must be %s characters') % num_chars)
        if sequence.prefix and not internal_number.startswith(sequence.prefix):
            errors.append(_("Force Number field must me start with '%s'") % sequence.prefix)
        if sequence.suffix and not internal_number.endswith(sequence.suffix):
            errors.append(_("Force Number field must me end with '%s'") % sequence.suffix)
        if errors:
            raise osv.except_osv(_('Error !'), "\n".join(errors))

    def create(self, cr, uid, data, context={}):
        if data.get('internal_number', False) and self._must_validate_internal_number(cr, uid, context=context):
            self._validate_internal_number(cr, uid, data['internal_number'], data['journal_id'], context)
        invoice_id = super(account_invoice, self).create(cr, uid, data, context=context)
        return invoice_id

    def write(self, cr, uid, ids, vals, context=None):
        result = super(account_invoice, self).write(cr, uid, ids, vals, context)
        if isinstance(ids, int):
            ids = [ids]
        for invoice in self.browse(cr, uid, ids, context=context):
            if invoice.type in ("out_invoice", "out_refund"):
                if invoice.internal_number and invoice.state == 'draft' \
                  and self._must_validate_internal_number(cr, uid, context=context):
                    self._validate_internal_number(cr, uid, invoice.internal_number, invoice.journal_id.id, context)
                # Hack for validate updates with Mass Editing module
                if context and 'mass_editing_object' in context \
                  and self._must_validate_internal_number(cr, uid, context=context):
                    if 'internal_number' in vals:
                        self._validate_internal_number(cr, uid, vals['internal_number'], \
                                vals.get('journal_id', False) or invoice.journal_id.id, context)
                    if 'number' in vals:
                        self._validate_internal_number(cr, uid, vals['number'], \
                                vals.get('journal_id', False) or invoice.journal_id.id, context)
        return result

account_invoice()
