# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, CRM Picking Extension
#    Copyright (C) 2014  Enterprise Objects Consulting
#
#    Authors: Mariano Ruiz <mrsarm@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv
from openerp.tools.translate import _

import netsvc

class stock_picking(osv.osv):
    _inherit = 'stock.picking'

    _columns = {
            'claim_id': fields.many2one('crm.claim',string='CRM Claim', select=True),
        }

    def onchange_get_address(self,cr,uid,ids,claim_id,context={}):
        v={}
        if claim_id:
            claim_obj=self.pool.get("crm.claim")
            claim=claim_obj.browse(cr,uid,claim_id,context=context)
            v['address_id']=claim.partner_id.address[0].id
        return {'value': v}

    def onchange_get_claim(self,cr,uid,ids,address_id,context={}):
        v={}
        claim_obj=self.pool.get("crm.claim")
        claim_ids=claim_obj.search(cr,uid,[('partner_address_id','=',address_id)],order='create_date')
        if claim_ids:
            v['claim_id']=claim_ids[len(claim_ids)-1]
        else:
            v['claim_id']=0
        return {'value': v}

    def create(self, cr, uid, data, context=None):
        claim_pick = False
        if 'claim_id' in data and 'origin' not in data and 'stage_type' in context:
            claim_pick = True
            # If the move is created manually from CRM Claim then picking type
            # is determined according locations used
            loc_obj = self.pool.get("stock.location")
            location = False
            location_dest = False
            if 'location_id' in data and 'location_dest_id' in data:
                location = loc_obj.browse(cr, uid, data['location_id'], context=context)
                location_dest = loc_obj.browse(cr, uid, data['location_dest_id'], context=context)
            elif len(data.get('move_lines', [])) and len(data['move_lines'][0]) > 2:
                location = loc_obj.browse(cr, uid, data['move_lines'][0][2]['location_id'], context=context)
                location_dest = loc_obj.browse(cr, uid, data['move_lines'][0][2]['location_dest_id'], context=context)
            if location and location_dest:
                if location_dest.repair_location or location.usage in ('procurement', 'customer', 'supplier'):
                    data['type'] = 'in'
                elif location.repair_location or location_dest.scrap_location or location_dest.usage == 'customer':
                    data['type'] = 'out'
                if location.usage == 'customer' or location_dest.usage == 'customer':
                    claim = self.pool.get("crm.claim").browse(cr, uid, data['claim_id'], context=context)
                    if not 'partner_id' in data:
                        data['partner_id'] = claim.partner_id.id
                    if not 'address_id' in data:
                        data['address_id'] = claim.partner_address_id.id

        picking_id = super(stock_picking, self).create(cr, uid, data, context=context)

        if claim_pick:
            wf_service = netsvc.LocalService("workflow")
            # Confirm and validate the picking
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
            self.action_move(cr, uid, [picking_id])
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_done', cr)

        return picking_id

stock_picking()


class stock_return_picking(osv.osv_memory):
    _inherit = 'stock.return.picking'

    def fields_view_get(self, cr, uid, view_id=None, view_type='form',
                        context=None, toolbar=False, submenu=False):

        record_id = context and context.get('active_id', False) or False
        res = super(stock_return_picking, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,submenu=submenu)
        active_model = context.get('active_model')
        if active_model != 'stock.picking':
            return res
        stock_pick = self.pool.get('stock.picking').browse(cr, uid, record_id, context=context)
        if not stock_pick.claim_id and not self._allow_return_without_claim(cr, uid, context):
            raise osv.except_osv(_('Error'), _("Picking has no CRM Claim"))
        return res

    def _allow_return_without_claim(self, cr, uid, context={}):
        property_obj = self.pool.get('ir.property')
        prop_ids = property_obj.search(cr, uid, [('name', '=', 'stock_return_allow_without_claim')],context=context)
        if len(prop_ids)>0:
            return property_obj.read(cr,uid,prop_ids[0],['value_integer'],context=context)['value_integer'] != 0
        return True

stock_return_picking()
