# -*- coding: utf-8 -*-
##############################################################################
#
#  OpenERP, CRM Claim Serial Number Extension
#  Copyright (c) 2014  Enterprise Objects Consulting
#  All Rights Reserved.
#
#  Authors: Mariano Ruiz <mrsarm@gmail.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _

class crm_claim(osv.osv):
    _inherit = 'crm.claim'

    _columns = {
            'serial_id': fields.many2one('stock.serial', 'Serial Number',
                            help='Set the Serial Number only if the Reference is a Product'),
        }

    def create(self, cr, uid, data, context=None):
        if 'serial_id' in data and not 'ref' in data:
            # This is only to prevent a error in OpenERP 6.1, in some
            # circunstance only with reference fields setted with a on_change function,
            # value are not passed in form send, so in this case we can infer from
            # serial selected
            data['ref'] = "product.product,%s" % self.pool.get("stock.serial"). \
                    browse(cr,uid,data['serial_id'],context=context).product_id.id
        claim_id = super(crm_claim, self).create(cr, uid, data, context=context)
        return claim_id

    def write(self, cr, uid, ids, vals, context=None):
        if not isinstance(ids, (list, tuple)):
            ids = [ids]
        result = super(crm_claim, self).write(cr, uid, ids, vals, context=context)
        for claim in self.browse(cr, uid, ids, context):
            if claim.serial_id.id and not claim.ref.id:
                result = super(crm_claim, self).write(cr, uid, [claim.id], {"ref": "product.product,%s" % claim.serial_id.product_id.id}, context=context)
                claim = self.browse(cr, uid, claim.id, context)
            elif claim.serial_id.id and claim.ref._model._name != 'product.product':
                raise osv.except_osv(_('Error !'),
                        _('If Serial Number is configured then Reference must be a Product !'))
            if claim.serial_id.id and claim.ref.id and claim.ref.id != claim.serial_id.product_id.id:
                raise osv.except_osv(_('Error !'),
                        _('Serial Number "%s" is not from "%s" Product !') % (claim.serial_id.name, claim.ref.name))
            if not claim.serial_id.id and claim.ref.id and claim.ref.is_serial:
                raise osv.except_osv(_('Error !'),
                        _('You must be select a Serial Number if Product is "%s"') % claim.ref.name)
        return result

    def _prepare_order_moves_in(self, cr, uid, claim, picking_id, context={}):
        moves_data = super(crm_claim, self)._prepare_order_moves_in(cr, uid, claim, picking_id, context)
        if claim.serial_id:
            for move in moves_data:
                if move['product_id'] == claim.ref.id:
                    move['serial_ids'] = [(4,claim.serial_id.id)]
                    break
        return moves_data

    def _prepare_order_moves_out(self, cr, uid, claim, picking_id, move_lines_in, context={}):
        moves_data = super(crm_claim, self)._prepare_order_moves_out(cr, uid, claim, picking_id, move_lines_in, context)
        if claim.serial_id:
            for move in moves_data:
                move_origin = False
                for move_in in move_lines_in:
                    if move_in['id'] == move['move_origin_return']:
                        move_origin = self.pool.get("stock.move").browse(cr, uid, move_in['id'], context=context)
                        break
                if move_origin and len(move_origin.serial_ids):
                    move['serial_ids'] = [(4,serial.id) for serial in move_origin.serial_ids]
        return moves_data

    def on_change_serial(self, cr, uid, claim_id, serial_id, ref, context={}):
        res = {'value': {'serial_id': serial_id}}
        if serial_id and (ref and 'product.product' not in ref):
            raise osv.except_osv(_('Error !'), _("Set the Serial Number only if you have set a Product as Reference"))
        if serial_id:
            serial = self.pool.get('stock.serial').browse(cr, uid, serial_id, context=context)
            res['value']['ref'] = 'product.product,%s' % serial.product_id.id
            if serial.last_address_id.id:
                res['value']['partner_address_id'] = serial.last_address_id.id
                res['value']['partner_id'] = serial.last_address_id.partner_id.id
                res['value']['email_from'] = serial.last_address_id.email
                res['value']['partner_phone'] = serial.last_address_id.phone
            else:
                res['value']['partner_address_id'] = False
                res['value']['partner_id'] = False
                res['value']['email_from'] = False
                res['value']['partner_phone'] = False
        return res

crm_claim()
