# -*- coding: utf-8 -*-
##############################################################################
#
#  OpenERP, CRM Claim Simple Extension
#  Copyright (c) 2014  Enterprise Objects Consulting
#  All Rights Reserved.
#
#  Authors: Mariano Ruiz <mrsarm@gmail.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
from datetime import datetime, timedelta
from openerp.tools.translate import _

AVAILABLE_ACTIONS = [
        ('correction','Corrective Action'),
        ('prevention','Preventive Action'),
        ('replace','Replace Action'),    # New option
        ('discard','Discard Action'),    # New option
    ]

class crm_claim(osv.osv):
    _inherit = 'crm.claim'

    _columns = {
            'date_deadline': fields.datetime('Deadline', required=True,
                            states={'cancel':[('readonly',True)],
                                    'done':[('readonly',True)]}),  # Override field type, required and readonly
            'date': fields.datetime('Claim Date', select=True,
                            readonly=True, states={'draft':[('readonly',False)]}),      # Override readonly
            'description': fields.text('Description', required=True,
                            readonly=True, states={'draft':[('readonly',False)]}),      # Override readonly and required
            'categ_id': fields.many2one('crm.case.categ', 'Category', \
                            domain="[('section_id','=',section_id),\
                            ('object_id.model', '=', 'crm.claim')]", required=True,
                            states={'cancel':[('readonly',True)],
                                    'done':[('readonly',True)]}),       # Override required and readonly
            'type_action': fields.selection(AVAILABLE_ACTIONS, 'Action Type',
                    states={'draft':[('readonly',True)],
                            'cancel':[('readonly',True)],
                            'done':[('readonly',True)]}),    # Override required and selections
            'cause': fields.text('Root Cause',
                            states={'draft':[('readonly',True)],
                                    'cancel':[('readonly',True)],
                                    'done':[('readonly',True)]}),       # Override readonly
            'resolution': fields.text('Resolution',
                            states={'draft':[('readonly',True)],
                                    'cancel':[('readonly',True)],
                                    'done':[('readonly',True)]}),       # Override readonly
            'user_id': fields.many2one('res.users', 'Responsible',
                            states={'cancel':[('readonly',True)],
                                    'done':[('readonly',True)]}),       # Override readonly
            'create_uid': fields.many2one('res.users', 'Creator',
                            ondelete='set null', readonly=True),
        }

    _defaults = {
            'date_deadline': lambda self, cr, uid, context: \
                    (datetime.now() + timedelta(days=1)).strftime('%Y-%m-%d %H:%M:%S'),
        }

    def create(self, cr, uid, vals, context=None):
        crm_id = super(crm_claim, self).create(cr, uid, vals, context)
        template_obj = self.pool.get("email.template")
        template_id = template_obj.search(cr, uid, [("model_id.model","=","crm.claim")], context=context)
        if len(template_id)>0:
            template_obj.send_mail(cr, uid,template_id[0], crm_id, context=context)
        return crm_id

    def write(self, cr, uid, ids, vals, context=None):
        result = super(crm_claim, self).write(cr, uid, ids, vals, context=context)
        if vals.get('resolution', False):
            for claim in self.read(cr, uid, ids, ['type_action'], context=context):
                if not claim['type_action']:
                    raise osv.except_osv(_('Error !'), _('You must be specify an Action Type !'))
        return result

    def case_close(self, cr, uid, ids, *args):
        for l in self.browse(cr, uid, ids):
            if not l.type_action or not l.resolution or not l.cause:
                raise osv.except_osv(_('Error !'),
                        _('You must be specify a Root Cause, an Action Type and a Resolution to close the order !'))
        res = super(crm_claim, self).case_close(cr, uid, ids, *args)
        return res

crm_claim()
