# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Quotation Report
#    Copyright (C) 2014 Enterprise Objects Consulting
#                       http://www.eoconsulting.com.ar
#    Authors: Mariano Ruiz <mrsarm@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
import logging

_logger = logging.getLogger(__name__)

class sale_order(osv.osv):
    _inherit = "sale.order"

    def _default_days_validity(self, cr, uid, context=None):
        property_obj = self.pool.get('ir.property')
        prop_ids = property_obj.search(cr, uid, [('name', '=', 'sale_order_default_days_validity')],context=context)
        try:
            if len(prop_ids)>0:
                default_days_validity = property_obj.read(cr,uid,prop_ids[0],['value_integer'],context=context)['value_integer']
                return int(default_days_validity)
        except ValueError:
            _logger.error("Invalid integer value '%s' for 'sale_order_default_days_validity' key" % default_days_validity or '')
        return 45

    def _default_note(self, cr, uid, context=None):
        property_obj = self.pool.get('ir.property')
        prop_ids = property_obj.search(cr, uid, [('name', '=', 'sale_order_default_note')],context=context)
        if len(prop_ids)>0:
            default_note = property_obj.read(cr,uid,prop_ids[0],['value_text'],context=context)['value_text']
            return default_note
        return None


    _columns = {
            'days_validity': fields.integer("Days Validity", help="Days until which the quotation is regarded valid",
                                          readonly=True, states={'draft': [('readonly', False)]}),
        }

    _defaults = {
            #
            # For previous orders, you can run this SQL script to update the field:
            #
            #     UPDATE sale_order SET days_validity = 45 WHERE days_validity IS NULL;
            #
            'days_validity': _default_days_validity,
            'note': _default_note,
        }

sale_order()
