# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Stock Repair Location
#    Copyright (C) 2013 Mariano Ruiz <mrsarm@gmail.com>
#    Enterprise Objects Consulting (<http://www.eoconsulting.com.ar>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv

class stock_location(osv.osv):
    _inherit = "stock.location"

    _columns = {
        'repair_location': fields.boolean('Repair location'),
    }
    _defaults = {
        'repair_location': False,
    }

    def get_default_repair_location(self, cr, uid, warehouse_id=None, context={}):
        """
        Return a default repair location for any operation
        """
        lot_repair_id = None
        if warehouse_id:
            lot_repair_id = self.pool.get("stock.warehouse") \
                   .browse(cr,uid,warehouse_id,context=context).lot_repair_id.id
        if not lot_repair_id:
            lot_repair_id = self.get_repair_location_from_ir_prop(cr, uid, context)
            if not lot_repair_id:
                loc_ids = self.search(cr,uid,[('repair_location','=',True)],context=context)
                if len(loc_ids):
                    lot_repair_id = loc_ids[0]
        return lot_repair_id

    def get_repair_location_from_ir_prop(self, cr, uid, context={}):
        property_obj = self.pool.get('ir.property')
        prop_ids = property_obj.search(cr, uid, [('name', '=', 'stock_return_repair')],context=context)
        if len(prop_ids)>0:
            return property_obj.read(cr,uid,prop_ids[0],['value_text'],context=context)['value_text']
        return None

stock_location()


class stock_warehouse(osv.osv):
    _inherit = "stock.warehouse"

    _columns = {
        'lot_repair_id': fields.many2one('stock.location', 'Location Repair',
                domain=[('usage','<>','view'),('repair_location','=',True)]),
    }

stock_warehouse()


class stock_move(osv.osv):
    _inherit = "stock.move"

    def _create_product_valuation_moves(self, cr, uid, move, context=None):
        if not move.location_id.repair_location and not move.location_dest_id.repair_location:
            super(stock_move,self)._create_product_valuation_moves(cr, uid, move, context)

stock_move()
